@@ -1,5 +1,6 @@
 import falcon
 import falcon.testing as testing
+import six
 
 
 class IDResource(object):
@@ -46,6 +47,29 @@ class TestUriTemplates(testing.TestBase):
         self.assertRaises(TypeError, self.api.add_route, set(), self.resource)
         self.assertRaises(TypeError, self.api.add_route, self, self.resource)
 
+    def test_field_name_cannot_start_with_digit(self):
+        self.assertRaises(ValueError, self.api.add_route,
+                          '/hello/{1world}', self.resource)
+
+        self.assertRaises(ValueError, self.api.add_route,
+                          '/{524hello}/world', self.resource)
+
+    def test_whitespace_not_allowed(self):
+        self.assertRaises(ValueError, self.api.add_route,
+                          '/{thing }/world', self.resource)
+
+        self.assertRaises(ValueError, self.api.add_route,
+                          '/{ thing}/world', self.resource)
+
+        self.assertRaises(ValueError, self.api.add_route,
+                          '/{ thing }/world', self.resource)
+
+        self.assertRaises(ValueError, self.api.add_route,
+                          '/{thing}/wo rld', self.resource)
+
+        self.assertRaises(ValueError, self.api.add_route,
+                          '/{thing} /world', self.resource)
+
     def test_no_vars(self):
         self.api.add_route('/hello/world', self.resource)
         self.simulate_request('/hello/world')
@@ -55,6 +79,18 @@ class TestUriTemplates(testing.TestBase):
 
         self.assertEqual(req.get_param('world'), None)
 
+    def test_unicode_literal_routes(self):
+        if six.PY3:
+            self.skipTest('Test only applies to Python 2')
+
+        self.api.add_route(u'/hello/world', self.resource)
+        self.simulate_request('/hello/world')
+
+        self.assertTrue(self.resource.called)
+        req = self.resource.req
+
+        self.assertEqual(req.get_param('world'), None)
+
     def test_special_chars(self):
         self.api.add_route('/hello/world.json', self.resource)
         self.api.add_route('/hello(world)', self.resource)
